Begin["TQPAT`Private`"];

(*Loading LoopTools*)
Install[$LoopToolsPath];
DB0L = B0i[dbb0, ##1] &;
B0L = B0i[bb0, ##1] &;
DownValues[C0] /. C0 -> C0L /. RuleDelayed -> SetDelayed;
DownValues[A0] /. A0 -> A0L /. RuleDelayed -> SetDelayed;
Remove[A0,B0,DB0,C0];
LoopToolsNumercis = {Global`A0 -> A0L, Global`B0 -> B0L, Global`DB0 -> DB0L, Global`C0 -> C0L};

(*Loads a string from file*)
LoadFileString[FilePath_String] := Module[{FileString, NewLine, FullFileStream},
   FullFileStream = OpenRead[FilePath];
   FileString = "";
   While[(NewLine = Read[FullFileStream, String]) =!= EndOfFile, 
    FileString = FileString <> "\n" <> NewLine;];
   FileString = StringDrop[FileString,1];
   Close[FullFileStream];
   Return[FileString];
   ];

(*Writing a string to file*)
WriteFileString[FileString_String, FilePath_String] := Module[{FileStream},
   FileStream = OpenWrite[FilePath];
   WriteString[FileStream, FileString];
   Close[FileStream];
   ];

(*Rewrite Tag SumOver into nested SumOver Functions*)
PrepareForVertexNumerics[expr_]:=Module[{temp,SumOverList},
temp=expr;
SumOverList=Union[Cases[temp,SumOver[a__],{0,Infinity}]];

temp=Collect[temp,SumOver[a___],Tag];
While[Length[SumOverList]>0,
temp=Collect[temp,SumOverList[[1]]]/.SumOverList[[1]]*a_:>Append[SumOverList[[1]],a];
SumOverList=Drop[SumOverList,1];];
temp=temp/.Tag[a__]:>Collect[Cancel[Together[a]],{A0[c___],B0[c___],C0[c___]},(*Full*)Simplify];
Return[temp];
];


(*Replacing PaVe Integrals by analytic results*)
ExpressPaVeFunctions[expr_,option_]:=Reap[Module[{PreviousDefinitions,n},

(*Saving PreviousDefinitions*)
PreviousDefinitions={DownValues[A0],DownValues[B0],DownValues[C0],DownValues[D0]};

(*Analytic Continuation Prescription*)
Global`Numerica=1;

If[option===Normal, Get[ToFileName[{$TQPATProgramDir,"integraltables"},"PaVeFunctionsSpecial.m"]];];
If[option===Debug, Get[ToFileName[{$TQPATProgramDir,"integraltables"},"PaVeFunctionsSpecialDebug.m"]];];
If[option===Finite, Get[ToFileName[{$TQPATProgramDir,"integraltables"},"PaVeFunctionsSpecialFinite.m"]];];
If[option===Divergent, Get[ToFileName[{$TQPATProgramDir,"integraltables"},"PaVeFunctionsSpecialDivergent.m"]];];
If[option===Expanded, Get[ToFileName[{$TQPATProgramDir,"integraltables"},"PaVeFunctionsSpecialExpanded.m"]];];

Sow[expr];
Clear[A0,A0Case1,B0,B0Case1,B0Case2,B0Case3,B0Case4,B0Case5,B0Case6,B0Case7,B0Case8,B0Case9,B0Case10,DB0,DB0Case1,DB0Case2,DB0Case3,DB0Case4,DB0Case5,DB0Case6,C0,C0Case1,C0Case2,C0Case3];

(*Restoring Previous Definitions if needed*)
For[n=1,n<=4,n++,
If[Length[PreviousDefinitions[[n]]]>0,
 ReleaseHold[PreviousDefinitions[[n]]]/.RuleDelayed->SetDelayed;]
];

]][[2]][[1]][[1]];


(*STD Model Quark Mixing Matrix*)
RemoveCKMMixing[expression_] := 
    expression /. CKMC -> CKM /. {CKM[1, 1] -> 1,
        CKM[1, 2] -> 0,
        CKM[1, 3] -> 0,
        CKM[2, 1] -> 0,
        CKM[2, 2] -> 1,
        CKM[2, 3] -> 0,
        CKM[3, 1] -> 0,
        CKM[3, 2] -> 0,
        CKM[3, 3] -> 1};


(*Susy Mixings like Stop1 and Stop2 Mixing for a = 3, b = 3*)
PluginRealSfermionMixingMatrix[expression_] := 
    expression /. {USfC -> USf, VSfC -> VSf}/. {USf[1, 1, a_, b_] :> Cos[Global`theta[a, b]], 
        USf[1, 2, a_, b_] :> Sin[Global`theta[a, b]], 
        USf[2, 1, a_, b_] :> -Sin[Global`theta[a, b]], 
        USf[2, 2, a_, b_] :> Cos[Global`theta[a, b]]}/.{VSf[1, 1, a_, b_] :> Cos[Global`theta[a, b]], 
        VSf[1, 2, a_, b_] :> Sin[Global`theta[a, b]], 
        VSf[2, 1, a_, b_] :> -Sin[Global`theta[a, b]], 
        VSf[2, 2, a_, b_] :> Cos[Global`theta[a, b]]};

(*Replace Functionelemtens by Monomials*)
ExplicitMasses[expression_]:= expression/.{
Mf[2, 1] -> Sqrt[ME2],  Mf[2, 2] -> Sqrt[MM2],  Mf[2, 3] -> Sqrt[ML2],
Mf[3, 1] -> Sqrt[MU2],  Mf[3, 2] -> Sqrt[MC2],  Mf[3, 3] -> Sqrt[MT2],
Mf[4, 1] -> Sqrt[MD2],  Mf[4, 2] -> Sqrt[MS2],  Mf[4, 3] -> Sqrt[MB2],
Mf2[2, 1] -> ME2,  Mf2[2, 2] -> MM2,  Mf2[2, 3] -> ML2,
Mf2[3, 1] -> MU2,  Mf2[3, 2] -> MC2,  Mf2[3, 3] -> MT2,
Mf2[4, 1] -> MD2,  Mf2[4, 2] -> MS2,  Mf2[4, 3] -> MB2,
MSf[1,1,3] -> MSnl1, MSf[2,1,3] -> MSnl2,
MSf[1,1,2] -> MSnm1, MSf[2,1,2] -> MSnm2,
MSf[1,1,1] -> MSne1, MSf[2,1,1] -> MSne2,
MSf[1,2,3] -> MSl1, MSf[2,2,3] -> MSl2,
MSf[1,2,2] -> MSm1, MSf[2,2,2] -> MSm2,
MSf[1,2,1] -> MSe1, MSf[2,2,1] -> MSe2,
MSf[1,3,3] -> MSt1, MSf[2,3,3] -> MSt2,
MSf[1,3,2] -> MSc1, MSf[2,3,2] -> MSc2,
MSf[1,3,1] -> MSu1, MSf[2,3,1] -> MSu2,
MSf[1,4,3] -> MSb1, MSf[2,4,3] -> MSb2,
MSf[1,4,2] -> MSs1, MSf[2,4,2] -> MSs2,
MSf[1,4,1] -> MSd1, MSf[2,4,1] -> MSd2,
MSf2[1,1,3] -> MSnl12, MSf2[2,1,3] -> MSnl22,
MSf2[1,1,2] -> MSnm12, MSf2[2,1,2] -> MSnm22,
MSf2[1,1,1] -> MSne12, MSf2[2,1,1] -> MSne22,
MSf2[1,2,3] -> MSl12, MSf2[2,2,3] -> MSl22,
MSf2[1,2,2] -> MSm12, MSf2[2,2,2] -> MSm22,
MSf2[1,2,1] -> MSe12, MSf2[2,2,1] -> MSe22,
MSf2[1,3,3] -> MSt12, MSf2[2,3,3] -> MSt22,
MSf2[1,3,2] -> MSc12, MSf2[2,3,2] -> MSc22,
MSf2[1,3,1] -> MSu12, MSf2[2,3,1] -> MSu22,
MSf2[1,4,3] -> MSb12, MSf2[2,4,3] -> MSb22,
MSf2[1,4,2] -> MSs12, MSf2[2,4,2] -> MSs22,
MSf2[1,4,1] -> MSd12, MSf2[2,4,1] -> MSd22,
MNeu[1] -> MNeu1, MNeu[2] -> MNeu2, MNeu[3] -> MNeu3, MNeu[4] -> MNeu4,
MNeu2[1] -> MNeu12, MNeu2[2] -> MNeu22, MNeu2[3] -> MNeu32, MNeu2[4] -> MNeu42,
MCha[1] -> MCha1, MCha[2] -> MCha2,
MCha2[1] -> MCha12, MCha2[2] -> MCha22};

(*Macht ExplicitMasses rueckgaengig*)
UndoExplicitMasses[expression_] := 
 expression /.{ME -> Mf[2, 1], MM -> Mf[2, 2], ML -> Mf[2, 3], 
   MU -> Mf[3, 1], MC -> Mf[3, 2], MT -> Mf[3, 3], MD -> Mf[4, 1], 
   MS -> Mf[4, 2], MB -> Mf[4, 3], ME2 -> Mf2[2, 1], MM2 -> Mf2[2, 2],
    ML2 -> Mf2[2, 3], MU2 -> Mf2[3, 1], MC2 -> Mf2[3, 2], 
   MT2 -> Mf2[3, 3], MD2 -> Mf2[4, 1], MS2 -> Mf2[4, 2], 
   MB2 -> Mf2[4, 3], MSnl1 -> MSf[1, 1, 3], MSnl2 -> MSf[2, 1, 3], 
   MSnm1 -> MSf[1, 1, 2], MSnm2 -> MSf[2, 1, 2], 
   MSne1 -> MSf[1, 1, 1], MSne2 -> MSf[2, 1, 1], MSl1 -> MSf[1, 2, 3],
    MSl2 -> MSf[2, 2, 3], MSm1 -> MSf[1, 2, 2], MSm2 -> MSf[2, 2, 2], 
   MSe1 -> MSf[1, 2, 1], MSe2 -> MSf[2, 2, 1], MSt1 -> MSf[1, 3, 3], 
   MSt2 -> MSf[2, 3, 3], MSc1 -> MSf[1, 3, 2], MSc2 -> MSf[2, 3, 2], 
   MSu1 -> MSf[1, 3, 1], MSu2 -> MSf[2, 3, 1], MSb1 -> MSf[1, 4, 3], 
   MSb2 -> MSf[2, 4, 3], MSs1 -> MSf[1, 4, 2], MSs2 -> MSf[2, 4, 2], 
   MSd1 -> MSf[1, 4, 1], MSd2 -> MSf[2, 4, 1], 
   MSnl12 -> MSf2[1, 1, 3], MSnl22 -> MSf2[2, 1, 3], 
   MSnm12 -> MSf2[1, 1, 2], MSnm22 -> MSf2[2, 1, 2], 
   MSne12 -> MSf2[1, 1, 1], MSne22 -> MSf2[2, 1, 1], 
   MSl12 -> MSf2[1, 2, 3], MSl22 -> MSf2[2, 2, 3], 
   MSm12 -> MSf2[1, 2, 2], MSm22 -> MSf2[2, 2, 2], 
   MSe12 -> MSf2[1, 2, 1], MSe22 -> MSf2[2, 2, 1], 
   MSt12 -> MSf2[1, 3, 3], MSt22 -> MSf2[2, 3, 3], 
   MSc12 -> MSf2[1, 3, 2], MSc22 -> MSf2[2, 3, 2], 
   MSu12 -> MSf2[1, 3, 1], MSu22 -> MSf2[2, 3, 1], 
   MSb12 -> MSf2[1, 4, 3], MSb22 -> MSf2[2, 4, 3], 
   MSs12 -> MSf2[1, 4, 2], MSs22 -> MSf2[2, 4, 2], 
   MSd12 -> MSf2[1, 4, 1], MSd22 -> MSf2[2, 4, 1], MNeu1 -> MNeu[1], 
   MNeu2 -> MNeu[2], MNeu3 -> MNeu[3], MNeu4 -> MNeu[4], 
   MNeu12 -> MNeu2[1], MNeu22 -> MNeu2[2], MNeu32 -> MNeu2[3], 
   MNeu42 -> MNeu2[4], MCha1 -> MCha[1], MCha2 -> MCha[2], 
   MCha12 -> MCha2[1], MCha22 -> MCha2[2]};

(*Mit Dieser Funktion lassen sich die y-Werte mehrere Datenreihen addieren:
Format einer Datenreihe ist D1={{x1,y1},{x2,y2},...,{xn,yn}}
fuer a kann D1,D2,...,Dn eingesetzt werden, wobei die Laenge der Datenreihen natuerlich gleich sein muss!! Ausserdem sollten die Datenreihen die gleichen x-Werte besitzen!*)
AddyValues[a___] := Module[{ListOpList},
   ListOpList[c_, f_, b_] := Module[{i}, i = 1; Return[({c[[i]][[1]], f[c[[i++]][[2]], #[[2]]]}) & /@ b];];
   If[Length[List[a]] == 0,
    Return["Nothing to Add!"];];
   If[Length[List[a]] == 1,
    Return[a];];
   If[Length[List[a]] >= 2,
    Return[AddyValues[Sequence @@ List[ListOpList[List[a][[1]], Plus, List[a][[2]]]],Sequence @@ Drop[List[a], 2]]];];
   ];

(*Diese Funktion kann auf DataSets angewendet werden, um Unter- oder Ueberschreitungen  von Schranken auszuwerten*)
(*Bsp fuer Eingabeformat fuer die Schranken:
PATBoundaryConditionList = {{Mh0, 75, 80}, {MA0, 100, Infinity}, {MZ2,-Infinity, 1}}
*)
(*Bsp fuer Outputformat: 
{{100, {Mh0 -> BelowLowerBound[3]}}, 
 {45, {Mh0 -> AboveUpperBound[85], MA0 -> BelowLowerBound[22]}}, 
 {60, {}}}
*)

CheckBoundaryConditions[DataSet_, BoundaryConditionList_] := Module[{CheckDataPoint, CheckBounds, UpperBound, LowerBound},
   UpperBound[Param_] := Cases[BoundaryConditionList, {Param, a_, b_} :> b][[1]];
   LowerBound[Param_] := Cases[BoundaryConditionList, {Param, a_, b_} :> a][[1]];
   CheckBounds[Param_, Number_] := Which[Number > (UpperBound[Param]), Rule[Param, AboveUpperBound[Number]], 
     Number < (LowerBound[Param]), Rule[Param, BelowLowerBound[Number]], 
     Number <= (UpperBound[Param]) && Number >= (LowerBound[Param]), NZSequence[]];

   CheckDataPoint[DataPoint_] :={DataPoint[[1]], 
(Cases[DataPoint, Rule[#[[1]], a_] :> CheckBounds[#[[1]], a], {0, Infinity}, Heads -> True] /. NZSequence -> Sequence /. List -> Sequence )& /@ BoundaryConditionList};

   Return[CheckDataPoint[#] & /@ DataSet]
   ];

ReplaceValuesInDataSet[DataSet_, ListOfNewRules_List] := DataSet /. ((Rule[#[[1]], a_] :>Rule[#[[1]], (#[[2]] /.Global`PreviousValue :> a)]) & /@ ListOfNewRules);


(*Different Correction types*)
CorrectionQ = MemberQ[{HeliL, HeliR, DeltaC, Direct}, #1] &;
NumericProcedureQ=MemberQ[{SENumerics, tVertexNumerics, eVertexNumerics, BoxNumerics, ExtrapolationNumerics}, #1]&;
(*Checks existence of ListOfContributions element. First it checks the existence of all short forms (HeliL,HeliR,DeltaC).. only number prefactors are allowed! For Direct numerics no exsistence check is done!*)
ElementListOfContributionsQ = ((Head[Contribution @@ ((Cases[#, HeliL[b_, c_Symbol] | HeliR[b_, c_Symbol] | DeltaC[b_, a___, c_Symbol], {0, Infinity}]) /. List[] -> List[MissingContribution[]])[[1]]] =!= Contribution && MatchQ[#, (x_?CorrectionQ[b_Symbol, d___, c_Symbol]) | (Times[a_?NumberQ, x_?CorrectionQ[b_Symbol, d___, c_Symbol]])]) || MatchQ[#, (Direct[b_, c_?NumericProcedureQ]) | (Times[a_?NumberQ, Direct[b_, c_?NumericProcedureQ]])]) &;

(*This Function creates numbers from analytic expressions and Data \Sets*)
Options[DoAllNumerics]={Global`AddUpAllContributions->True};
DoAllNumerics[ListOfContributions_, NumericData_,ops:OptionsPattern[]] :=Module[{Result, TempResult, GetXValues, GetYValues, ComposeXYTuple, ListOpList, DifferentNumericRules, NumericRules, ContributionRules, FilteredList},

IndexDelta[a_Integer, b_Integer] := KroneckerDelta[a, b];
GetXValues[NumericData2_] := #[[1]] & /@ NumericData2;
GetYValues[NumericData2_] := #[[2]] & /@ NumericData2;
ComposeXYTuple[XValues_, YValues_] := Module[{i}, i = 1; Return[{#, YValues[[i++]]} & /@ XValues]];
ListOpList[a_, f_, b_] := Module[{i}, i = 1; Return[({a[[i]][[1]], f[a[[i++]][[2]], #[[2]]]}) & /@ b];];

(*Hier wird die richte Auswertemethode in Abhaengigkeit des Korrekturtyps gewaehlt*)
DifferentNumericRules = {TQPAT`TreeL->BoxNumerics, TQPAT`TreeR->BoxNumerics, TQPAT`GammaGamma -> SENumerics,TQPAT`GammaZ -> SENumerics, TQPAT`ZZ -> SENumerics, TQPAT`Ztt -> tVertexNumerics, TQPAT`Gammatt -> tVertexNumerics, TQPAT`eeZL -> eVertexNumerics, TQPAT`eeZR -> eVertexNumerics, TQPAT`eeGammaL -> eVertexNumerics, TQPAT`eeGammaR -> eVertexNumerics, TQPAT`BOXL -> BoxNumerics, TQPAT`BOXR -> BoxNumerics, TQPAT`Extrapolation -> ExtrapolationNumerics};

(*Hier werden die Beitraege mit dem richtigen Vorfaktor und derrichtigen Auswertemethode bestimmt*)

ContributionRules = {DeltaC[ParticleSector_, options___, CorrectionType_] :> (CorrectionType /. DifferentNumericRules) @@ {ToCrossectionCorrection[CorrectionType]*Contribution[ParticleSector, options, CorrectionType],NumericData},
HeliL[ParticleSector_, options___, Extrapolation] :> (Print["Only relative corrections to the cross section can be extrapolated! Setting extrapolation contributions to zero!"];(Table[0,{i,Length[NumericData]}])),
HeliL[ParticleSector_, CorrectionType_] :> (CorrectionType /.DifferentNumericRules) @@ {ToHelicityAmplitude[CorrectionType,L]*Contribution[ParticleSector, CorrectionType], NumericData},
HeliR[ParticleSector_, options___, Extrapolation] :> (Print["Only relative corrections to the cross section can be extrapolated! Setting extrapolation contributions to zero!"];(Table[0,{i,Length[NumericData]}])),
HeliR[ParticleSector_, CorrectionType_] :> (CorrectionType /. DifferentNumericRules) @@ {ToHelicityAmplitude[CorrectionType,R]*Contribution[ParticleSector, CorrectionType], NumericData},
Direct[AnalyticExpression2_, options___, Extrapolation] :> (Print["Only relative corrections to the cross section can be extrapolated! Setting extrapolation contributions to zero!"];(Table[0,{i,Length[NumericData]}])),
Direct[AnalyticExpression2_, NumericType_] :> NumericType @@{AnalyticExpression2, NumericData}};

(*Definition der verschiedenen Auswertmethoden*)

NumericRules = {SENumerics[AnalyticExpression_, NumericData2_] :> ((Re[ExpressPaVeFunctions[ExplicitMasses[(AnalyticExpression //. SumOver[a_, b_, c_] :> Sum[c, {a, 1, b}])] /. Dispatch[#], Finite]]) & /@ GetYValues[NumericData2]), 
    tVertexNumerics[AnalyticExpression_, NumericData2_] :> ((Re[ExpressPaVeFunctions[ExplicitMasses[RemoveCKMMixing[(AnalyticExpression //.SumOver[a_, b_, c_] :> Sum[c, {a, 1, b}])]] /. Dispatch[#], Finite]]) & /@ GetYValues[NumericData2]), 
    eVertexNumerics[AnalyticExpression_, NumericData2_] :> ((Re[(ExplicitMasses[RemoveCKMMixing[(AnalyticExpression //.SumOver[a_, b_, c_] :> Sum[c, {a, 1, b}])]] /.Dispatch[#]) /. LoopToolsNumercis]) & /@ GetYValues[NumericData2]), 
    BoxNumerics[AnalyticExpression_, NumericData2_] :> ((Re[ExpressPaVeFunctions[ExplicitMasses[RemoveCKMMixing[(AnalyticExpression //.SumOver[a_, b_, c_] :> Sum[c, {a, 1, b}])]] /.Dispatch[#], Finite]]) & /@ GetYValues[NumericData2]),
    ExtrapolationNumerics[{PreProgramName_, options___}, NumericData2_] :> ((DeltaSigmaExtrapolation[#, ToExpression[StringDrop[ToString[PreProgramName],2]], options]) & /@GetYValues[NumericData2])};

(*Argument Filter*)
FilteredList=If[Head[ListOfContributions]=!=List,{ListOfContributions},ListOfContributions];
FilteredList=(If[ElementListOfContributionsQ[#], #, Print[ToString[#] <> " is not defined and will be dropped from ListOfContributions!"];Sequenced[], #] & /@ FilteredList) /. Sequenced -> Sequence;
If[Length[FilteredList]===0,Return["ListOfContributions is empty!"];];

(*Eigentliche Auswertung*)
TempResult=FilteredList/.ContributionRules/.NumericRules;

(*Bei der nächsten numerischen Auswertung Daten überschreiben*)
RestartExtrapolationCounter;

(*Hier wird die x-Komponente wieder eingebaut*)
Result = ComposeXYTuple[GetXValues[NumericData], #] & /@ TempResult;
TempResult = Result;

(*Breruecksichtigung der Summationsoption*)
If[OptionValue[Global`AddUpAllContributions],
Result = TempResult[[1]];
While[Length[TempResult] > 1,
Result = ListOpList[Result, Plus, TempResult[[2]]];
TempResult = Drop[TempResult, {1}];];];
Return[Result];
];


Options[CreateSimpleDataSet] = {Global`VariableParameter ->Global`x, Global`ParameterMin -> 100, Global`ParameterMax -> 1000, Global`NumberOfPoints -> 26};

CreateSimpleDataSet[PreDataSet_, SubstitutionRule_,opt : OptionsPattern[]] := Module[{DeltaBetweenPoints, TempDataSet, DataSet, ParameterIntern, ParameterMinIntern, ParameterMaxIntern, NumberOfPointsIntern},
(*Definition of parametrisation*)
   ParameterIntern = OptionValue[Global`VariableParameter];
   ParameterMinIntern = OptionValue[Global`ParameterMin];
   ParameterMaxIntern = OptionValue[Global`ParameterMax];
   NumberOfPointsIntern = OptionValue[Global`NumberOfPoints];
(*Get Length of an interval*)
   DeltaBetweenPoints = Abs[(ParameterMaxIntern - ParameterMinIntern)]/(NumberOfPointsIntern - 1);
   
(*Replacing the Variables in PreDataSet according to SubstitutionRule*)
   TempDataSet = PreDataSet[[2]] /. ((Rule[#[[1]], DummyParameter_] -> Rule[#[[1]], #[[2]]]) & /@ SubstitutionRule);
   
(*Final creation of the full DataSets in dependence of the ParameterX*)
   DataSet = Table[{x, TempDataSet /. ParameterIntern -> x}, {x, ParameterMinIntern, ParameterMaxIntern, DeltaBetweenPoints}];
   Return[DataSet];];



SelectDataFromList[DataSet_List, SelectionList_List] := Module[{ReturnList},
   ReturnList = {};
(*Hier werden zwei verschiedenen Formate unterstuezzt:*)
(*1. DataSet: {{1,{a->1,...}},...}*)
   If[Head[DataSet] === List && Head[DataSet[[1]]] === List,
    (If[MemberQ[SelectionList, Head[#[[1]]]] || MemberQ[SelectionList, #[[1]]],
         ReturnList = Append[ReturnList, #];]) & /@ DataSet[[1]][[2]];];
(*2. RulesList: {a->1,b->2,...}*)
   If[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
    (If[MemberQ[SelectionList, Head[#[[1]]]] || MemberQ[SelectionList, #[[1]]],
         ReturnList = Append[ReturnList, #];]) & /@ DataSet;];
(*Der Output ist gleich: Liste Von Zahlen*)
   Return[ReturnList];
];

ExcludeDataByList[DataSet_List, SelectionList_List] := Module[{ReturnList},
   ReturnList = {};
(*Hier werden zwei verschiedenen Formate unterstuezzt:*)
(*1. DataSet: {{1,{a->1,...}},...}*)
   If[Head[DataSet] === List && Head[DataSet[[1]]] === List,
    (If[!MemberQ[SelectionList, Head[#[[1]]]] && !MemberQ[SelectionList, #[[1]]],
         ReturnList = Append[ReturnList, #];]) & /@ DataSet[[1]][[2]];];
(*2. RulesList: {a->1,b->2,...}*)
   If[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
    (If[!MemberQ[SelectionList, Head[#[[1]]]] && !MemberQ[SelectionList, #[[1]]],
         ReturnList = Append[ReturnList, #];]) & /@ DataSet;];
(*Der Output ist gleich: Liste Von Zahlen*)
   Return[ReturnList];
];

GetNumbersFromRulesList[DataSet_] := Module[{},
   If[Head[DataSet] === List && Head[DataSet[[1]]] === Rule, 
    Return[#[[2]] & /@ (DataSet)];]
   ];

GetNamesFromRulesList[DataSet_] := Module[{},
   If[Head[DataSet] === List && Head[DataSet[[1]]] === Rule, 
    Return[#[[1]] & /@ (DataSet)];]
   ];

(*Selecttiert aus einem sortiertem DataSet oder RulesList eine List von Zahlen (MSSM Parameter) in normierter Reihenfolge*)
CreateRunMathMSSMNumberListOld[DataSet_List] := 
  Module[{UsedVariableList, NumberList},
   UsedVariableList = {USf, ZNeu, UCha, VCha, Mf, MHH, MHp, Mh0, MA0, MGl, TB, SA, MSf, MCha, MNeu};
   NumberList = GetNumbersFromRulesList[SelectDataFromList[UndoExplicitMasses[DataSet], UsedVariableList]];
   Return[NumberList];
   ];

(*Selecttiert aus einem sortiertem DataSet oder RulesList eine List von Variablennamen (MSSM Parameter) in normierter Reihenfolge*)
(*Mit einem STD DataSet kann daraus die ListOfMSSMVariables erzeugt werden*)
CreateRunMathMSSMVariableList[DataSet_List] := 
  Module[{UsedVariableList, VariableList},
   UsedVariableList = {USf, ZNeu, UCha, VCha, Mf, MHH, MHp, Mh0, MA0, MGl, TB, SA, MSf, MCha, MNeu};
   VariableList = GetNamesFromRulesList[SelectDataFromList[UndoExplicitMasses[DataSet], UsedVariableList]];
   Return[VariableList];
   ];

(*Selecttiert aus einem sortiertem DataSet oder RulesList eine List von Zahlen in normierter (SM Parameter) Reihenfolge*)
CreateRunMathSMNumberListOld[DataSet_List] := 
  Module[{UsedVariableList, NumberList},
   UsedVariableList = {Alfa, Alfas, MW2, MZ2, Mf2};
   NumberList = GetNumbersFromRulesList[SelectDataFromList[UndoExplicitMasses[DataSet], UsedVariableList]];
   Return[NumberList];
   ];

(*Selecttiert aus einem sortiertem DataSet oder RulesList eine List von Variablennamen normierter (SM Parameter) Reihenfolge*)
(*Mit einem STD DataSet kann daraus die ListOfSMVariables erzeugt werden*)
CreateRunMathSMVariableList[DataSet_List] := 
  Module[{UsedVariableList, VariableList},
   UsedVariableList = {Alfa, Alfas, MW2, MZ2, Mf2};
   VariableList = GetNamesFromRulesList[SelectDataFromList[UndoExplicitMasses[DataSet], UsedVariableList]];
   Return[VariableList];
   ];


(*This order of MSSM parameters is used for the FOTRAN mathlink interface*) 
$TQPATOrderedMSSMVariableList = {MA0, MCha1, MCha2, MGl, Mh0, MHH,
   MHp, MNeu1, MNeu2, MNeu3, MNeu4, MSb1, MSb2, MSc1, MSc2, MSd1,
   MSd2, MSe1, MSe2, MSl1, MSl2, MSm1, MSm2, MSne1, MSnl1, MSnm1,
   MSs1, MSs2, MSt1, MSt2, MSu1, MSu2, SA, TB, UCha[1, 1], UCha[1, 2],
    UCha[2, 1], UCha[2, 2], USf[1, 1, 2, 1], USf[1, 1, 2, 2],
   USf[1, 1, 2, 3], USf[1, 1, 3, 1], USf[1, 1, 3, 2], USf[1, 1, 3, 3],
    USf[1, 1, 4, 1], USf[1, 1, 4, 2], USf[1, 1, 4, 3],
   USf[1, 2, 2, 1], USf[1, 2, 2, 2], USf[1, 2, 2, 3], USf[1, 2, 3, 1],
    USf[1, 2, 3, 2], USf[1, 2, 3, 3], USf[1, 2, 4, 1],
   USf[1, 2, 4, 2], USf[1, 2, 4, 3], USf[2, 1, 2, 1], USf[2, 1, 2, 2],
    USf[2, 1, 2, 3], USf[2, 1, 3, 1], USf[2, 1, 3, 2],
   USf[2, 1, 3, 3], USf[2, 1, 4, 1], USf[2, 1, 4, 2], USf[2, 1, 4, 3],
    USf[2, 2, 2, 1], USf[2, 2, 2, 2], USf[2, 2, 2, 3],
   USf[2, 2, 3, 1], USf[2, 2, 3, 2], USf[2, 2, 3, 3], USf[2, 2, 4, 1],
    USf[2, 2, 4, 2], USf[2, 2, 4, 3], VCha[1, 1], VCha[1, 2],
   VCha[2, 1], VCha[2, 2], ZNeu[1, 1], ZNeu[1, 2], ZNeu[1, 3],
   ZNeu[1, 4], ZNeu[2, 1], ZNeu[2, 2], ZNeu[2, 3], ZNeu[2, 4],
   ZNeu[3, 1], ZNeu[3, 2], ZNeu[3, 3], ZNeu[3, 4], ZNeu[4, 1],
   ZNeu[4, 2], ZNeu[4, 3], ZNeu[4, 4]};

(*This order of SM parameters is used for the FOTRAN mathlink interface*) 
$TQPATOrderedSMVariableList = {Alfa, Alfas, MB2, MC2, MD2, ME2, ML2, MM2, MS2, MT2, MU2, MW2, MZ2};

(*The new version can treat unordered DataSet files*)
CreateRunMathMSSMNumberList[DataSet_] := Module[{OrderedNumberList},

Which[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
      OrderedNumberList = $TQPATOrderedMSSMVariableList /.DataSet /.$TQPATMSSMDefaultValues;,
      Head[DataSet] === List && Head[DataSet[[1]]] === List && Head[DataSet[[1, 2]]] === List && Head[DataSet[[1, 2, 1]]] === Rule,
      OrderedNumberList = $TQPATOrderedMSSMVariableList /. (DataSet[[1, 2]]) /. $TQPATMSSMDefaultValues;,
      True,
      Print["Input doesnt match required format!"];
      Return[$Failed];
     ];
If[OrderedNumberList =!= (OrderedNumberList /. TQPATDefault -> (# &)), Print["Undefined MSSM variable detected!\nUsing data from $TQPATMSSMDefaultValues for missing numerical values!"];];

Return[OrderedNumberList /. TQPATDefault -> (# &)];
];

(*The new version can treat unordered DataSet files*)
CreateRunMathSMNumberList[DataSet_] := Module[{OrderedNumberList},

Which[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
      OrderedNumberList = $TQPATOrderedSMVariableList /.DataSet /.$TQPATSMDefaultValues;,
      Head[DataSet] === List && Head[DataSet[[1]]] === List && Head[DataSet[[1, 2]]] === List && Head[DataSet[[1, 2, 1]]] === Rule,
      OrderedNumberList = $TQPATOrderedSMVariableList /. (DataSet[[1, 2]]) /. $TQPATSMDefaultValues;,
      True,
      Print["Input doesnt match required format!"];
      Return[$Failed];
     ];
If[OrderedNumberList =!= (OrderedNumberList /. TQPATDefault -> (# &)), Print["Undefined SM variable detected!\nUsing data from $TQPASMDefaultValues for missing numerical values!"];];

Return[OrderedNumberList /. TQPATDefault -> (# &)];
];


Set$TQPATMSSMDefaultValues[DataSet_] := Module[{n, OrderedNumberList, DefaultValueList},

   Which[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
         OrderedNumberList = ($TQPATOrderedMSSMVariableList /. DataSet );,
         Head[DataSet] === List && Head[DataSet[[1]]] === List && Head[DataSet[[1, 2]]] === List && Head[DataSet[[1, 2, 1]]] === Rule,
         OrderedNumberList = ($TQPATOrderedMSSMVariableList /. (DataSet[[1, 2]]));,
         True,
         Print["Input doesnt match required format!"]; Return[$Failed];
        ];

   n = 1;
   DefaultValueList = Rule[#, OrderedNumberList[[n++]]] & /@ $TQPATOrderedMSSMVariableList;
   If[And @@ (DefaultValueList /. Rule -> UnsameQ),
      Print["Updating $TQPATMSSMDefaultValues..."];
      $TQPATMSSMDefaultValues = (DefaultValueList /. Rule[a_, b_] :> Rule[a, TQPATDefault[b]]);,
      Print["Missing MSSM variables detected!\nCannot update $TQPATMSSMDefaultValues with given input!"]; Return[$Failed];
     ];
];

Set$TQPATSMDefaultValues[DataSet_] := Module[{n, OrderedNumberList, DefaultValueList},

   Which[Head[DataSet] === List && Head[DataSet[[1]]] === Rule,
         OrderedNumberList = ($TQPATOrderedSMVariableList /. DataSet );,
         Head[DataSet] === List && Head[DataSet[[1]]] === List && Head[DataSet[[1, 2]]] === List && Head[DataSet[[1, 2, 1]]] === Rule,
         OrderedNumberList = ($TQPATOrderedSMVariableList /. (DataSet[[1, 2]]));,
         True,
         Print["Input doesnt match required format!"]; Return[$Failed];
        ];

   n = 1;
   DefaultValueList = Rule[#, OrderedNumberList[[n++]]] & /@ $TQPATOrderedSMVariableList;
   If[And @@ (DefaultValueList /. Rule -> UnsameQ),
      Print["Updating $TQPATSMDefaultValues..."];
      $TQPATSMDefaultValues = (DefaultValueList /. Rule[a_, b_] :> Rule[a, TQPATDefault[b]]);,
      Print["Missing SM variables detected!\nCannot update $TQPATSMDefaultValues with given input!"]; Return[$Failed];
     ];
];





(*Dies Funktion Erzeug und Extrapoliert automatisch den Wert von Deltasigma an der Schwelle mit FORTRAN*)
(*Es sind folgenden Optionen moeglich:*)
(*OptionsList =
{
 IntervalBetweenInterpolationPoints -> 1, 
 NumberOfInterpolationPoints -> 10, 
 InterpolationType -> {Linear, Quadratic, Cubic, Interpolation}, 
 PlotInterpolation -> {1, 0}, 
 WriteInterpolationData -> {1, 0}
 FortranProgramName->"runmath"
};*)
(*Unique number String for labeling output file names *) 
RestartExtrapolationCounter := Remove[#] & /@ Names["ExtrapolationCounter*"];

NewExtrapolationNumber := LastExtrapolationNumber = StringDrop[ToString[Unique["ExtrapolationCounter"]], 20]; LastExtrapolationNumber;

ExtrapolationPlotRules = {Frame -> True,
   FrameLabel -> {Style[Global`s^(1/2), 18], Style[\[CapitalDelta], 18]},
   ImageSize -> 500,
   LabelStyle -> Directive[Italic], FrameStyle -> 16,
   PlotStyle -> {Black, Thick}};

Options[DeltaSigmaExtrapolation] := {Global`ExtrapolationDataDir -> $TQPATExtrapolationDataDir,
   Global`FortranDir -> $TQPATFortranDir,
   Global`NumberOfDataPoints -> 5,
   Global`DeltaBetweenDataPoints -> 1,
   Global`ExtrapolationType -> Global`Cubic,
   Global`PlotExtrapolation -> False,
   Global`WriteExtrapolationData -> False,
   (*Global`HardCodedSMData ->False,*)
   Sequence @@ (Complement[#, FilterRules[#, ExtrapolationPlotRules]] &[Options[Plot]]), Sequence @@ ExtrapolationPlotRules};

DeltaSigmaExtrapolation[DataSet_List, ProgramName_, opts : OptionsPattern[]] := Module[{ComposeXYTuple, TempFolder, TopMassSquared, SDataList, TreeDataListDependentOnS, OneLoopDataListDependentOnS, DeltaSigmaDependentOnS, n, PlotOfInterpolation, InterpolationFit, LinearFit, QuadraticFit, CubicFit, FitData, PlotOptionRules, CurrentOptionRules},

   CurrentOptionRules = (#[[1]] -> OptionValue[#[[1]]]) & /@Options[DeltaSigmaExtrapolation];
   PlotOptionRules = FilterRules[CurrentOptionRules, Options[Plot]];

   (*Erzeugt aus zwei Datenlisten ein XYTuple*)
   ComposeXYTuple[XValues_, YValues_] := Module[{i}, i = 1; Return[{#, YValues[[i++]]} & /@ XValues]];

   TempFolder = Directory[];
   SetDirectory[OptionValue[Global`FortranDir]];

   (*Bestimmung der Topquarkmasse*)
   If[Length[SelectDataFromList[DataSet, {MT2}]] =!= 0, 
    TopMassSquared = MT2 /. SelectDataFromList[DataSet, {MT2}];, 
    Print["Mass of Topquark is not defined in DataSet! Setting MT to 175 GeV."];
    TopMassSquared = 175^2;
   ];

   (*If[OptionValue[Global`HardCodedSMData],
   (ToExpression[ToString[ProgramName]<>"SMHC"]) @@ {{2*Sqrt[TopMassSquared] + 0.01, 2*Sqrt[TopMassSquared] + 0.01 + (OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints], OptionValue[Global`DeltaBetweenDataPoints]}, CreateRunMathMSSMNumberList[DataSet]};,*)
   ProgramName @@ {{2*Sqrt[TopMassSquared] + 0.01, 2*Sqrt[TopMassSquared] + 0.01 + (OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints], OptionValue[Global`DeltaBetweenDataPoints]}, CreateRunMathSMNumberList[DataSet], CreateRunMathMSSMNumberList[DataSet]};
   (*];*)

   (* In the case where the FORTRAN program didnt work:*)

  If[Head[XSection`Data[1]] === XSection`Data, Print["There was and error during the execution of the MathLink program: "<>ToString[ProgramName]]; Return[$Failed];];

   (*Einlesen der Datenreihen*)
   SDataList = (#[[1]][[1]]) & /@ XSection`Data[1];
   TreeDataListDependentOnS = (#[[2]][[1]]) & /@ XSection`Data[1];
   OneLoopDataListDependentOnS = (#[[2]][[2]]) & /@ XSection`Data[1];
   n = 1;

   Clear[XSection`Data];

   (*Berechnung von Delta Sigma*)
   DeltaSigmaDependentOnS = (#/TreeDataListDependentOnS[[n++]]) & /@ OneLoopDataListDependentOnS;

   (*Zusammenstellen der Fitdaten*)
   FitData = ComposeXYTuple[SDataList, DeltaSigmaDependentOnS];

   (*Extrapolation*) 
   Which[OptionValue[Global`ExtrapolationType] === Interpolation,
    InterpolationFit = Interpolation[FitData];,
    OptionValue[Global`ExtrapolationType] === Global`Linear,
    LinearFit = Fit[FitData, {1, x}, x];,
    OptionValue[Global`ExtrapolationType] === Global`Quadratic,
    QuadraticFit = Fit[FitData, {1, x, x^2}, x];,
    OptionValue[Global`ExtrapolationType] === Global`Cubic,
    CubicFit = Fit[FitData, {1, x, x^2, x^3}, x];
    ];

   (*Ausgabe des Interpolationsplots in eine Datei*)
   NewExtrapolationNumber;
   Which[OptionValue[Global`PlotExtrapolation] === True && OptionValue[Global`ExtrapolationType] === Interpolation,
    Print["Exporting " <> "Fit_" <> ToString[ProgramName] <> "_Interpolation_" <> LastExtrapolationNumber <> ".eps..."];
    Show[
     Plot[InterpolationFit[x], {x, 2*Sqrt[TopMassSquared] - 1, 2*Sqrt[TopMassSquared] +1+(OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints]}, AxesOrigin -> {2*Sqrt[TopMassSquared], InterpolationFit[2*Sqrt[TopMassSquared]]}, ##] &@(Sequence @@PlotOptionRules), 
     ListPlot[FitData, PlotStyle -> Directive[PointSize[Medium], Black]], 
     DisplayFunction -> (Export[OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Interpolation_" <> LastExtrapolationNumber <> ".eps", #] &)],

    OptionValue[Global`PlotExtrapolation] === True && OptionValue[Global`ExtrapolationType] === Global`Linear, 
    Print["Exporting " <> "Fit_" <> ToString[ProgramName] <> "_Linear_" <> LastExtrapolationNumber <> ".eps..."];
    Show[
     Plot[LinearFit, {x, 2*Sqrt[TopMassSquared] - 1, 2*Sqrt[TopMassSquared] + 1 + (OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints]}, AxesOrigin -> {2*Sqrt[TopMassSquared], LinearFit /.x -> 2*Sqrt[TopMassSquared]}, ##] &@(Sequence @@PlotOptionRules), 
     ListPlot[FitData, PlotStyle -> Directive[PointSize[Medium], Black]], 
     DisplayFunction -> (Export[OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Linear_" <> LastExtrapolationNumber <> ".eps", #] &)],

    OptionValue[Global`PlotExtrapolation] === True && OptionValue[Global`ExtrapolationType] === Global`Quadratic, 
    Print["Exporting " <> "Fit_" <> ToString[ProgramName] <>"_Quadratic_" <> LastExtrapolationNumber <> ".eps..."];
    Show[
     Plot[QuadraticFit, {x, 2*Sqrt[TopMassSquared] - 1, 2*Sqrt[TopMassSquared] + 1 + (OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints]}, AxesOrigin -> {2*Sqrt[TopMassSquared], QuadraticFit /.x -> 2*Sqrt[TopMassSquared]}, ##] &@(Sequence @@ PlotOptionRules), 
     ListPlot[FitData,PlotStyle -> Directive[PointSize[Medium], Black]], 
     DisplayFunction -> (Export[OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Quadratic_" <> LastExtrapolationNumber <> ".eps", #] &)],

    OptionValue[Global`PlotExtrapolation] === True && OptionValue[Global`ExtrapolationType] === Global`Cubic,
    Print[
     "Exporting " <> "Fit_" <> ToString[ProgramName] <> "_Cubic_" <> LastExtrapolationNumber <> ".eps..."];
    Show[
     Plot[CubicFit, {x, 2*Sqrt[TopMassSquared] - 1, 2*Sqrt[TopMassSquared] + 1 + (OptionValue[Global`NumberOfDataPoints] - 1)*OptionValue[Global`DeltaBetweenDataPoints]}, AxesOrigin -> {2*Sqrt[TopMassSquared], CubicFit /. x -> 2*Sqrt[TopMassSquared]}, ##] &@(Sequence @@PlotOptionRules), 
     ListPlot[FitData, PlotStyle -> Directive[PointSize[Medium], Black]], 
     DisplayFunction -> (Export[OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Cubic_" <> LastExtrapolationNumber <> ".eps", #] &)]];

   (*Ausgabe der Interpolationsdaten in Dateien*)
   If[OptionValue[Global`WriteExtrapolationData] === True,
    Print["Writing Fit_" <> ToString[ProgramName] <> "_Data_" <> LastExtrapolationNumber <> ".m..."];
    Put[FitData,OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Data_" <> LastExtrapolationNumber <> ".m"];
    Which[
     OptionValue[Global`ExtrapolationType] === Global`Linear,
     Print["Writing Fit_" <> ToString[ProgramName] <> "_Equation_Linear_" <>LastExtrapolationNumber <> ".m..."];
     Put[LinearFit, OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Equation_Linear_" <> LastExtrapolationNumber <> ".m"];,
     OptionValue[Global`ExtrapolationType] === Global`Quadratic,
     Print["Writing Fit_" <> ToString[ProgramName] <> "_Equation_Quadratic_" <> LastExtrapolationNumber <> ".m..."];
     Put[QuadraticFit, OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Equation_Quadratic_" <> LastExtrapolationNumber <> ".m"];,
     OptionValue[Global`ExtrapolationType] === Global`Cubic,
     Print["Writing Fit_" <> ToString[ProgramName] <> "_Equation_Cubic_" <> LastExtrapolationNumber <> ".m..."];
     Put[CubicFit, OptionValue[Global`ExtrapolationDataDir] <> "Fit_" <> ToString[ProgramName] <> "_Equation_Cubic_" <> LastExtrapolationNumber <> ".m"];]];

   (*Ins Urspruengliche Verzeichnis wechseln*)
   SetDirectory[TempFolder];

   (*Rueckgabe des Extrapolierten Wertes*)
   Which[OptionValue[Global`ExtrapolationType] === Global`Linear,
    Return[LinearFit /. x -> 2*Sqrt[TopMassSquared]];,
    OptionValue[Global`ExtrapolationType] === Global`Quadratic,
    Return[QuadraticFit /. x -> 2*Sqrt[TopMassSquared]];,
    OptionValue[Global`ExtrapolationType] === Global`Cubic,
    Return[CubicFit /. x -> 2*Sqrt[TopMassSquared]];,
    OptionValue[Global`ExtrapolationType] === Interpolation,
    Return[InterpolationFit[2*Sqrt[TopMassSquared]]];];
];



(*Interface to SPHENO*)
GetNeutralinoData[PathToSPhenoFile_]:=
Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"Neutralinos"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"e-sneutrino mass"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MNeu1->ToExpression[Data[[1]]<>"``15"],
MNeu2->ToExpression[Data[[2]]<>"``15"],
MNeu3->ToExpression[Data[[3]]<>"``15"],
MNeu4->ToExpression[Data[[4]]<>"``15"],
MNeu12->ToExpression[Data[[1]]<>"``15"]^2,
MNeu22->ToExpression[Data[[2]]<>"``15"]^2,
MNeu32->ToExpression[Data[[3]]<>"``15"]^2,
MNeu42->ToExpression[Data[[4]]<>"``15"]^2};
MixingData={};
For[i=1,i<=4,i++,
For[j=1,j<=4,j++,
MixingData=Append[MixingData,ZNeu[i,j]->Simplify[(ToExpression[Data[[-5+8*i+2*j]]<>"`5"]+I*ToExpression[Data[[-4+8*i+2*j]]<>"`5"])]];
MixingData=Append[MixingData,ZNeuC[i,j]->Simplify[(ToExpression[Data[[-5+8*i+2*j]]<>"`5"]-I*ToExpression[Data[[-4+8*i+2*j]]<>"`5"])]];
]
];
Return[Union[MassData,MixingData]];
];

GetCharginoData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"Charginos"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"Neutralinos"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile, "Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MCha1->ToExpression[Data[[1]]<>"``15"],
MCha2->ToExpression[Data[[2]]<>"``15"],
MCha12->ToExpression[Data[[1]]<>"``15"]^2,
MCha22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,UCha[i,j]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,UChaC[i,j]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,VCha[i,j]->Simplify[(ToExpression[Data[[2*i+j+4]]<>"`8"])]];
MixingData=Append[MixingData,VChaC[i,j]->Simplify[(ToExpression[Data[[2*i+j+4]]<>"`8"])]];
]
];
Return[Union[MassData,MixingData]];
];


GetSneutrinoData[PathToSPhenoFile_]:=
Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"e-sneutrino mass   :"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"selectron masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSne1->ToExpression[Data[[1]]<>"``15"],
MSnm1->ToExpression[Data[[2]]<>"``15"],
MSnl1->ToExpression[Data[[3]]<>"``15"],
MSne12->ToExpression[Data[[1]]<>"``15"]^2,
MSnm12->ToExpression[Data[[2]]<>"``15"]^2,
MSnl12->ToExpression[Data[[3]]<>"``15"]^2};
Return[MassData];];


GetSelectronData[PathToSPhenoFile_]:=
Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"selectron masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"smuon masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSe1->ToExpression[Data[[1]]<>"``15"],
MSe2->ToExpression[Data[[2]]<>"``15"],
MSe12->ToExpression[Data[[1]]<>"``15"]^2,
MSe22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,2,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,2,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
]
];
Return[Union[MassData,MixingData]];
];


GetSmuonData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"smuon masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"stau masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSm1->ToExpression[Data[[1]]<>"``15"],
MSm2->ToExpression[Data[[2]]<>"``15"],
MSm12->ToExpression[Data[[1]]<>"``15"]^2,
MSm22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,2,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,2,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
]
];
Return[Union[MassData,MixingData]];
];


GetStauData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"stau masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"u-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSl1->ToExpression[Data[[1]]<>"``15"],
MSl2->ToExpression[Data[[2]]<>"``15"],
MSl12->ToExpression[Data[[1]]<>"``15"]^2,
MSl22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,2,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,2,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];]];
Return[Union[MassData,MixingData]];];


GetSuQuarkData[PathToSPhenoFile_]:=
Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"u-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"c-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSu1->ToExpression[Data[[1]]<>"``15"],
MSu2->ToExpression[Data[[2]]<>"``15"],
MSu12->ToExpression[Data[[1]]<>"``15"]^2,
MSu22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,3,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,3,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];]];
Return[Union[MassData,MixingData]];];


GetScQuarkData[PathToSPhenoFile_]:=
Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"c-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"t-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSc1->ToExpression[Data[[1]]<>"``15"],
MSc2->ToExpression[Data[[2]]<>"``15"],
MSc12->ToExpression[Data[[1]]<>"``15"]^2,
MSc22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,3,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,3,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];]];
Return[Union[MassData,MixingData]];];


GetStopData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"t-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"d-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile, "Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSt1->ToExpression[Data[[1]]<>"``15"],
MSt2->ToExpression[Data[[2]]<>"``15"],
MSt12->ToExpression[Data[[1]]<>"``15"]^2,
MSt22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,3,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,3,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
]
];
Return[Union[MassData,MixingData]];
];


GetSdQuarkData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"d-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"s-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSd1->ToExpression[Data[[1]]<>"``15"],
MSd2->ToExpression[Data[[2]]<>"``15"],
MSd12->ToExpression[Data[[1]]<>"``15"]^2,
MSd22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,4,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,4,1]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];]];
Return[Union[MassData,MixingData]];];


GetSsQuarkData[PathToSPhenoFile_]:= Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"s-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"b-squark masses"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSs1->ToExpression[Data[[1]]<>"``15"],
MSs2->ToExpression[Data[[2]]<>"``15"],
MSs12->ToExpression[Data[[1]]<>"``15"]^2,
MSs22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,4,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,4,2]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];]];
Return[Union[MassData,MixingData]];];


GetSbottomData[PathToSPhenoFile_]:=Module[{stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"b-squark masses"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"m_A0, m_H+"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile, "Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MSb1->ToExpression[Data[[1]]<>"``15"],
MSb2->ToExpression[Data[[2]]<>"``15"],
MSb12->ToExpression[Data[[1]]<>"``15"]^2,
MSb22->ToExpression[Data[[2]]<>"``15"]^2};
MixingData={};
For[i=1,i<=2,i++,
For[j=1,j<=2,j++,
MixingData=Append[MixingData,USf[i,j,4,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
MixingData=Append[MixingData,USfC[i,j,4,3]->Simplify[(ToExpression[Data[[2*i+j]]<>"`8"])]];
]
];
Return[Union[MassData,MixingData]];
];


GetHiggsData[PathToSPhenoFile_]:=
Module[{temp,stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"m_A0, m_H+"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"Low energy constraints"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile, "Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MA0->ToExpression[Data[[1]]<>"``15"],
MHp->ToExpression[Data[[2]]<>"``15"],
Mh0->ToExpression[Data[[3]]<>"``15"],
MHH->ToExpression[Data[[4]]<>"``15"],
MA02->ToExpression[Data[[1]]<>"``15"]^2,
MHp2->ToExpression[Data[[2]]<>"``15"]^2,
Mh02->ToExpression[Data[[3]]<>"``15"]^2,
MHH2->ToExpression[Data[[4]]<>"``15"]^2};
MixingData={
SA->ToExpression[Data[[7]]<>"`8"],
CA->ToExpression[Data[[5]]<>"`8"],
SA2->ToExpression[Data[[7]]<>"`8"]^2,
CA2->ToExpression[Data[[5]]<>"`8"]^2};
temp=Union[MassData,MixingData];
MixingData={};
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"tan(beta) at m_Z :"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile, "Text"],"sign(mu)"]][[1]]-1;
RawData=StringTake[Import[PathToSPhenoFile, "Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MixingData={
TB->ToExpression[Data[[1]]<>"``15"],
TB2->ToExpression[Data[[1]]<>"``15"]^2,
SB->Sin[ArcTan[ToExpression[Data[[1]]<>"``15"]]],
SB2->Sin[ArcTan[ToExpression[Data[[1]]<>"``15"]]]^2,
CB->Cos[ArcTan[ToExpression[Data[[1]]<>"``15"]]],
CB2->Cos[ArcTan[ToExpression[Data[[1]]<>"``15"]]]^2};
Return[Union[temp,MixingData]];
];


GetGluinoData[PathToSPhenoFile_]:=
Module[{temp,stringpos1,stringpos2,RawData,Data,MassData,MixingData,i,j},
stringpos1=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"Gluino :"]][[2]]+1;
stringpos2=Flatten[StringPosition[Import[PathToSPhenoFile,"Text"],"Charginos"]][[1]]-1;

RawData=StringTake[Import[PathToSPhenoFile,"Text"],{stringpos1,stringpos2}];
Data=StringCases[RawData,RegularExpression["-*\\d+.\\d+"]];
MassData={
MGl->ToExpression[Data[[1]]<>"``15"],
MGl2->ToExpression[Data[[1]]<>"``15"]^2};
Return[MassData];
];

(*Transforms the Mathematica NumberFormat to SPheno Input Format: 1 -> 1.00000000E+00*)
SPhenoInputForm[x_] := StringReplace[ToString[ScientificForm[NumberForm[N[x], {9, 8}, NumberMultiplier -> " "], 9, NumberMultiplier -> " "]], {RegularExpression["\\s*[-](\\d\\d)\n([-]*\\d+[.]\\d+)\\s+\\d+"] :> "$2E-$1", 
  RegularExpression["\\s*[-](\\d)\n([-]*\\d+[.]\\d+)\\s+\\d+"] :> "$2E-0$1",
  RegularExpression["\\s*(\\d\\d)\n([-]*\\d+[.]\\d+)\\s+\\d+"] :> "$2E+$1",
  RegularExpression["\\s*(\\d)\n([-]*\\d+[.]\\d+)\\s+\\d+"] :> "$2E+0$1",
  RegularExpression["^([+]*[-]*\\d[.]\\d+)$"] :> "$1E+00", 
  __ -> "This number format is not supported!"}]


(*The std SM values (MT=175) agree with thise given in:
  A.~H.~Hoang and C.~J.~Reisser,
  ``On electroweak matching conditions for top pair production at threshold,''
  Phys. Rev. D {\bf 74} (2006) 034002
  [arXiv:hep-ph/0604104].
*)
Options[LoadSPhenoSpectrumData]={Global`Alfa->0.0072973525205055605,Global`CW2->0.7778763507772662,Global`MB2->17.64,Global`MC2->1.6900000000000002,Global`MD2->0.000025,Global`ME2->2.6112099999999997*^-7,Global`ML2->3.1684,Global`MM2->0.011236,Global`MS2->0.010000000000000002,Global`MU2->0.000025,Global`MW2->6468.180625,Global`MZ2->8315.178393760001,Global`SW2->0.22212364922273375,Global`Alfas->0.12,Global`MT2->30625}
LoadSPhenoSpectrumData[PathToSPhenoFile_,opt:OptionsPattern[]]:=Union[(#[[1]]->OptionValue[#[[1]]])&/@Options[LoadSPhenoSpectrumData],GetNeutralinoData[PathToSPhenoFile],GetCharginoData[PathToSPhenoFile],GetSneutrinoData[PathToSPhenoFile],GetSelectronData[PathToSPhenoFile],GetSmuonData[PathToSPhenoFile],GetStauData[PathToSPhenoFile],GetSuQuarkData[PathToSPhenoFile],GetScQuarkData[PathToSPhenoFile],GetStopData[PathToSPhenoFile],GetSdQuarkData[PathToSPhenoFile],GetSsQuarkData[PathToSPhenoFile],GetSbottomData[PathToSPhenoFile],GetHiggsData[PathToSPhenoFile],GetGluinoData[PathToSPhenoFile]];


(*This function gives back the numeric Replacementrules for mSugra-Data*) 
(*Point SPS1a defined as std Input*)
(*Options of LoadSPhenoSpectrumData are routed down to LoadSPhenoSpectrumData*)
Options[GetMsugraDataRules] = Union[Options[LoadSPhenoSpectrumData],{Global`m0 -> 100, Global`m12 -> 250, Global`A0 -> -100, Global`tanbeta -> 10, Global`signmu -> +1}];
GetMsugraDataRules[opt:OptionsPattern[]] := Module[{WorkingDirSave, ReplacementRules}, 
   WorkingDirSave = Directory[];
   SetDirectory[$SPhenoPath <> "/"];
   WriteFileString[
    StringReplace[LoadFileString[$SPhenoPatternInputFile],{
      "$m0" :> SPhenoInputForm[OptionValue[Global`m0]],
      "$m12" :> SPhenoInputForm[OptionValue[Global`m12]],
      "$tanbeta" :> SPhenoInputForm[OptionValue[Global`tanbeta]],
      "$signmu" :> SPhenoInputForm[OptionValue[Global`signmu]],
      "$A0" :> SPhenoInputForm[OptionValue[Global`A0]]}
    ],
    $SPhenoPath<>"/LesHouches.in"];
   Run["./SPheno"];
   ReplacementRules = LoadSPhenoSpectrumData[$SPhenoPath <> "/SPheno.out",Sequence @@(FilterRules[{opt},Options[LoadSPhenoSpectrumData]])];
   SetDirectory[WorkingDirSave];
   Return[ReplacementRules];];


(*This function creates MSSM parameter Sets with Spheno*)
(*STD Options create Slope SPS1 DataSet*)
(*Options of LoadSPhenoSpectrumData are routed down through GetMsugraDataRules to LoadSPhenoSpectrumData*)
Options[CreateDataSet] = List[Global`VariableParameter -> Global`x, Global`m0 -> 0.4*Global`x, Global`m12 -> Global`x, Global`A0 -> -0.4*Global`x, Global`tanbeta -> 10, Global`signmu -> 1, Global`ParameterMin -> 100, Global`ParameterMax -> 350, Global`NumberOfPoints -> 51,Sequence @@Options[LoadSPhenoSpectrumData]];
CreateDataSet[opt:OptionsPattern[]] := Module[{DeltaBetweenPoints, DataSet},
   (*Slopes*)
   If[OptionValue[Global`NumberOfPoints] > 1,
    (*Determine the length of one interval*) 
    DeltaBetweenPoints = Abs[(OptionValue[Global`ParameterMax] - OptionValue[Global`ParameterMin])]/(OptionValue[Global`NumberOfPoints] - 1);
    (*Creation of the Data Set*)
    DataSet = Table[{x,
       GetMsugraDataRules[
        Global`m0 -> (OptionValue[Global`m0]/.OptionValue[Global`VariableParameter] -> x),
        Global`m12 -> (OptionValue[Global`m12]/.OptionValue[Global`VariableParameter] -> x),
        Global`A0 -> (OptionValue[Global`A0]/.OptionValue[Global`VariableParameter] -> x),
        Global`tanbeta -> (OptionValue[Global`tanbeta]/.OptionValue[Global`VariableParameter] -> x),
        Global`signmu -> (OptionValue[Global`signmu]/.OptionValue[Global`VariableParameter] -> x),
        Sequence @@(FilterRules[{opt},Options[LoadSPhenoSpectrumData]])]},
       {x, OptionValue[Global`ParameterMin], OptionValue[Global`ParameterMax], DeltaBetweenPoints}];
   ];
   (*Points*)
   (*For points ParameterMin replaces the VariableParameter*)
   If[OptionValue[Global`NumberOfPoints] === 1,
    DataSet = {{OptionValue[Global`ParameterMin],
       GetMsugraDataRules[
        Global`m0 -> (OptionValue[Global`m0] /.OptionValue[Global`VariableParameter] ->OptionValue[Global`ParameterMin]),
        Global`m12 -> (OptionValue[Global`m12] /.OptionValue[Global`VariableParameter] ->OptionValue[Global`ParameterMin]),
        Global`A0 -> (OptionValue[Global`A0] /.OptionValue[Global`VariableParameter] ->OptionValue[Global`ParameterMin]),
        Global`tanbeta -> (OptionValue[Global`tanbeta] /.OptionValue[Global`VariableParameter] ->OptionValue[Global`ParameterMin]),
        Global`signmu -> (OptionValue[Global`signmu] /.OptionValue[Global`VariableParameter] ->OptionValue[Global`ParameterMin]),
        Sequence @@(FilterRules[{opt},Options[LoadSPhenoSpectrumData]])]}};
    ];
   Return[DataSet];];


(*To Display all DataSets in a folder. Standard folder is: .../TQPAT/TQPAT/dataset.*)
Options[DisplayDataSets] = {Path ->StringDrop[$TQPATDataSetDir,-1]};
DisplayDataSets[opt : OptionsPattern[]] := StringReplace[#, {OptionValue[Path] <> "/" -> "",OptionValue[Path] -> "", ".set" -> ""}] & /@FileNames["*.set", OptionValue[Path]];

(*Load a DataSet from file into Mathematica variable. Standard folder is: : .../TQPAT/TQPAT/dataset.*)
Options[LoadDataSet] := {Global`VariableName -> Global`Standard, Global`Silent->False, Sequence @@ (Options[DisplayDataSets])};
LoadDataSet[DataSetName_String, opt : OptionsPattern[]] :=If[MemberQ[DisplayDataSets[Path -> OptionValue[Path]], DataSetName],
     If[(# =!= ToString[ToExpression[#]]) &@(OptionValue[Global`VariableName] /. Global`Standard -> DataSetName),
       Print["Cannot overwrite existing variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataSetName) <>". Use the VariableName option to specify a different variable name!"];,
       If[(Length[UpValues[#]] === Length[DownValues[#]] === 0) &@(ToExpression[OptionValue[Global`VariableName] /.Global`Standard -> DataSetName]),
         Evaluate[ToExpression[OptionValue[Global`VariableName] /. Global`Standard -> DataSetName]] = Get[OptionValue[Path] <> "/" <> DataSetName <> ".set"];
         If[!OptionValue[Global`Silent],Print[DataSetName <> " loaded into variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataSetName) <> "!"];];,
         Print["Cannot assign value to variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataSetName) <> " due to existing Up- and/or DownValues. Use the VariableName option to specify an other variable name!"];
         ];
       ];,
     Print["Cannot find DataSet!"];
    ];

(*Save a DataSet to File*)
SetAttributes[SaveDataSet, HoldFirst];
Options[SaveDataSet] := {FileName -> Global`Standard, Global`Silent->False, Sequence @@ (Options[DisplayDataSets])};
SaveDataSet[DataSetVariable_, opt : OptionsPattern[]] := Module[{FileNameI},
   If[SetDirectory[OptionValue[Path]] =!= $Failed,
     FileNameI = (OptionValue[FileName] /.Global`Standard -> StringDrop[StringDrop[ToString[If[Head[DataSetVariable]=!=Hold,Hold[DataSetVariable],DataSetVariable]], 5], -1]) <> ".set";
     Put[ReleaseHold[DataSetVariable], FileNameI];
     ResetDirectory[];
     If[!OptionValue[Global`Silent],
     Print[StringDrop[StringDrop[ToString[If[Head[DataSetVariable]=!=Hold,Hold[DataSetVariable],DataSetVariable]], 5], -1] <>" has been stored in the file " <> FileNameI <> "."];
     ];,
     Print["Target directory does not exist!"];
     ];
   ];

(*Delete a DataSet File*)
Options[DeleteDataSetFile] := Options[DisplayDataSets];
DeleteDataSetFile[FileNameX_String, opt : OptionsPattern[]] :=If[MemberQ[DisplayDataSets[Path -> OptionValue[Path]], FileNameX],
   SetDirectory[OptionValue[Path]];
   DeleteFile[FileNameX <> ".set"];
   Print[FileNameX <> ".set has been deleted!"];
   ResetDirectory[];,
   Print["File not found!"];
   ];

(*To Display all DataResults in a folder. Standard folder is: .../TQPAT/TQPAT/dataresults.*)
Options[DisplayDataResults] = {Path ->StringDrop[$TQPATDataResultDir,-1]};
DisplayDataResults[opt : OptionsPattern[]] := StringReplace[#, {OptionValue[Path] <> "/" -> "",OptionValue[Path] -> "", ".res" -> ""}] & /@FileNames["*.res", OptionValue[Path]];

(*Save DataResults to File*)
SetAttributes[SaveDataResult, HoldFirst];
Options[SaveDataResult] := {FileName -> Global`Standard, Global`Silent->False, Sequence @@ (Options[DisplayDataResults])};
SaveDataResult[DataResultVariable_, opt : OptionsPattern[]] := Module[{FileNameI},
   If[SetDirectory[OptionValue[Path]] =!= $Failed,
     FileNameI = (OptionValue[FileName] /.Global`Standard -> StringDrop[StringDrop[ToString[If[Head[DataResultVariable]=!=Hold,Hold[DataResultVariable],DataResultVariable]], 5], -1]) <> ".res";
     Put[ReleaseHold[DataResultVariable], FileNameI];
     ResetDirectory[];
     If[!OptionValue[Global`Silent],
      Print[StringDrop[StringDrop[ToString[If[Head[DataResultVariable]=!=Hold,Hold[DataResultVariable],DataResultVariable]], 5], -1] <>" has been stored in the file " <> FileNameI <> "."];
     ],
     Print["Target directory does not exist!"];
     ];
   ];

(*Load a DataResult from file into Mathematica variable. Standard folder is: : .../TQPAT/TQPAT/dataresults.*)
Options[LoadDataResult] := {Global`VariableName -> Global`Standard, Global`Silent->False, Sequence @@ (Options[DisplayDataResults])};
LoadDataResult[DataResultName_String, opt : OptionsPattern[]] :=If[MemberQ[DisplayDataResults[Path -> OptionValue[Path]], DataResultName],
     If[(# =!= ToString[ToExpression[#]]) &@(OptionValue[Global`VariableName] /. Global`Standard -> DataResultName),
       Print["Cannot overwrite existing variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataResultName) <>". Use the VariableName option to specify a different variable name!"];,
       If[(Length[UpValues[#]] === Length[DownValues[#]] === 0) &@(ToExpression[OptionValue[Global`VariableName] /.Global`Standard -> DataResultName]),
         Evaluate[ToExpression[OptionValue[Global`VariableName] /. Global`Standard -> DataResultName]] = Get[OptionValue[Path] <> "/" <> DataResultName <> ".res"];
         If[!OptionValue[Global`Silent],Print[DataResultName <> " loaded into variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataResultName) <> "!"];];,
         Print["Cannot assign value to variable " <> (OptionValue[Global`VariableName] /. Global`Standard -> DataResultName) <> " due to existing Up- and/or DownValues. Use the VariableName option to specify an other variable name!"];
         ];
       ];,
     Print["Cannot find DataResult!"];
    ];

(*Delete a DataResult File*)
Options[DeleteDataResultFile] := Options[DisplayDataResults];
DeleteDataResultFile[FileNameX_String, opt : OptionsPattern[]] :=If[MemberQ[DisplayDataResults[Path -> OptionValue[Path]], FileNameX],
   SetDirectory[OptionValue[Path]];
   DeleteFile[FileNameX <> ".res"];
   Print[FileNameX <> ".res has been deleted!"];
   ResetDirectory[];,
   Print["File not found!"];
   ];

(*automatically creates a list of Names for Elements of contribution list*)
SetAttributes[ToDataResultName, HoldAll];
ToDataResultName[ContributionList_, DataSet_Symbol] := (ToString[Unevaluated[DataSet]] <> (# /.d_?CorrectionQ[a_, b_] :> ToString[d] <> ToString[b] <> ToString[a])) & /@ ContributionList;

(*Automatically creates and saves DataResults for a given ContributionList and DataSet into math kernel variables *)
SetAttributes[CreateDataSetResults, HoldAll];
Options[CreateDataSetResults] = {Global`Silent -> False};
CreateDataSetResults[ContributionList_, DataSet_Symbol, opt : OptionsPattern[]] := If[And @@ ((ToString[ToExpression[#]] === #) & /@ ToDataResultName[ContributionList, DataSet]), 
   Evaluate[ToExpression[ToDataResultName[ContributionList, DataSet]]] = (If[! OptionValue[Global`Silent], Print["Calculating " <> ToString[#[[1]]] <> " ..."];]; DoAllNumerics[#[[1]], ReleaseHold[#[[2]]]]) & /@ ((List @@ #) & /@ (Times @@{ContributionList, Hold[DataSet]})); If[! OptionValue[Global`Silent], Print[ToString[Unevaluated[ContributionList]] <> " has been calculated using " <> ToString[Unevaluated[DataSet]] <> "."];];, 
   Print["Cannot overwrite existing variables!\nUse ClearDataSetResultVariables[" <> ToString[Unevaluated[ContributionList]] <> ", " <> ToString[Unevaluated[DataSet]] <> "] to clear the needed variables!"];];

(*Clears all DataSetResultVarlables of a given ContributionList and DataSet from the math kernel*)
SetAttributes[ClearDataSetResultVariables, HoldAll];
Options[ClearDataSetResultVariables] = {Global`Silent -> False};
ClearDataSetResultVariables[ContributionList_, DataSet_Symbol, opt : OptionsPattern[]] := ((ToExpression["Clear[" <> # <> "]"]; If[! OptionValue[Global`Silent], Print[# <> " cleared!"];];) & /@ ToDataResultName[ContributionList, DataSet];);

(*Saves all DataResult files of a given ContributionList and DataSet into .res files*)
SetAttributes[SaveDataSetResults, HoldAll];
Options[SaveDataSetResults] := Options[SaveDataResult];
SaveDataSetResults[ContributionList_, DataSet_Symbol,opts : OptionsPattern[]] := (SaveDataResult[#, FilterRules[{opts}, Options[SaveDataResult]]] & /@ ToExpression[("Hold[" <> # <> "]") & /@ ToDataResultName[ContributionList, DataSet]];);


(*Loads all DataResult files of a given ContributionList and DataSet*)
SetAttributes[LoadDataSetResults, HoldAll];
Options[LoadDataSetResults] := Options[LoadDataResult];
LoadDataSetResults[ContributionList_, DataSet_Symbol, opts : OptionsPattern[]] :=(LoadDataResult[#, FilterRules[{opts}, Options[LoadDataResult]]] & /@ ToDataResultName[ContributionList, DataSet];);


(*Deletes all DataResult files of a given ContributionList and DataSet*)
SetAttributes[DeleteDataSetResultFiles, HoldAll];
Options[DeleteDataSetResultFiles] := Options[DeleteDataResultFile];
DeleteDataSetResultFiles[ContributionList_, DataSet_Symbol, opts : OptionsPattern[]] :=(DeleteDataResultFile[#, FilterRules[{opts}, Options[DeleteDataResultFile]]] & /@ ToDataResultName[ContributionList, DataSet];);

(*Used to add up y-Values of DataResults*)
SumUpDataResults[{a___} | a___] := Module[{n, PlusList, xValues, yValues},
   If[Length[{a}] === 1, Return[a];];
   If[Level[{a}, {3}] === {}, Return[{a}];];
   PlusList = Plus[a];
   xValues = List[a][[1]];
   n = 1;
   xValues = (If[OddQ[n++], #, Sequenced[]] & /@ Level[xValues, {2}]) /. Sequenced -> Sequence;
   n = 1;
   yValues = (If[EvenQ[n++], #, Sequenced[]] & /@ Level[PlusList, {2}]) /. Sequenced -> Sequence;
   n = 1;
   Return[{xValues[[n++]], #} & /@ yValues];
   ];


(*Used for the Interpolation of sums of DataResults*)
SetAttributes[CalcInterpolation, HoldAll];
CalcInterpolation[ContributionList_, DataSet_Symbol] := Interpolation[SumUpDataResults @@ ToExpression[ToDataResultName[ContributionList, DataSet]]];


(*Information functions*)
Options[DisplayListOfContributions] = {Global`Output -> String};
DisplayListOfContributions[opt : OptionsPattern[]] := Module[{ListOfAllContributions, ContributionTypes, ContributionContents, ContributionListing, n, ContributionListingString},
   ListOfAllContributions = Union[Reverse[#] & /@ ToExpression[StringCases[LoadFileString[$TQPATProgramDir<>"contributions.m"], RegularExpression["Contribution[[]([^]_]+\\s*[^]_]+)[]]"] :> "{$1}", Overlaps -> False]]];
   ContributionTypes = Union[#[[1]] & /@ ListOfAllContributions];
   ContributionContents = (#[[2]] & /@ Cases[ListOfAllContributions, {#, a_}, {0, Infinity}]) & /@ ContributionTypes;
   n = 1;
   ContributionListing = {ContributionTypes[[n++]], #} & /@ ContributionContents;
   ContributionListingString = ("ContributionType:\tContributionContent\n" <> StringReplace[ToString[ContributionListing], {"}," -> "\n", ", {" -> ":\t", " {" -> "", "{" -> "", "}" -> ""}]);
   Switch[OptionValue[Global`Output], 
    String, 
    Return[ContributionListingString];, 
    List, 
    Return[ContributionListing];
   ];
   ];

Options[DisplayContributionInformation] = {Global`ContributionType -> Global`NA, Global`ContributionContent -> Global`NA};
DisplayContributionInformation[opt : OptionsPattern[]] := Module[{ListOfInformation, FoundInformation, n},
   If[(OptionValue[Global`ContributionType] === Global`NA) || (OptionValue[Global`ContributionContent] === Global`NA),
    Return["Missing ContributionType or ContributionContent!\nUsage example:\nDisplayContributionInformation[ContributionType->Gammatt, ContributionContent->_]\nUse DisplayListOfContributions[] to get a compact overview of all ContributionTypes and ContributionContents."];
    ];
   ListOfInformation = Sort[ToExpression[#] & /@ StringCases[LoadFileString[$TQPATProgramDir<>"contributions.m"], RegularExpression["[(][*]([{][^}]*[}])[*][)]"] :> "$1"]]//. {a___, {c_, d_, f_}, {c_, d_, g_}, b___} :> {a, {c, d, f <> "\n" <> g}, b};
   FoundInformation = Cases[ListOfInformation, {OptionValue[Global`ContributionContent],OptionValue[Global`ContributionType], casespattern_}];
   If[Length[FoundInformation] === 0,
    Return["No information found for Contribution[" <> ToString[OptionValue[Global`ContributionContent]] <> ", " <> ToString[OptionValue[Global`ContributionType]] <> "].\nUse DisplayListOfContributions[] to get a compact list of all ContributionTypes and ContributionContents."];,
    Return[StringJoin @@ (("Contribution[" <> ToString[#[[1]]] <> ", " <> ToString[#[[2]]] <> "] " <> #[[3]] <> "\n\n") & /@ FoundInformation)];
    ];
   ];

(*Used in LabelContourPlot. Merges two lists: AppendValue[{{x,y},{g,h}},{{1,2},{3,4}}]={{x,y,1,2},{g,h,3,4}} *)
AppendValue[ListOfLists_List, AppendList_] := Module[{n}, n = 1; Return[{Sequence @@ #, Sequence @@ AppendList[[n++]]} & /@ ListOfLists];]


(*Defines the layout of labels used in LabelContourPlot*)
TQPATLabelFunction = Text[Framed[Style[#[[3]], Bold, 16, Italic, FontColor -> Darker[Blue]], FrameMargins -> 3, FrameStyle -> Black], {#[[1]], #[[2]]}, Background -> White] &;

(*Helps to create proper Lables in ContourPlots using interactive interface -> only working in math notebooks!! *)
Options[LabelContourPlot] = {Global`LabelPrecision -> 2, Global`LabelFunction ->TQPATLabelFunction};
LabelContourPlot[CountourPlotHolded_Hold, Opts : OptionsPattern[]] := Module[{PlotSettings, PlotFunction, x, xmin, xmax, y, ymin, ymax, STDLocatorPosition},

   PlotSettings = ReleaseHold[CountourPlotHolded /. ContourPlot -> List];
   PlotFunction = PlotSettings[[1]];
   {x, xmin, xmax} = PlotSettings[[2]];
   {y, ymin, ymax} = PlotSettings[[3]];
   STDLocatorPosition = {(xmin + xmax)/2, (ymin + ymax)/2};

   Return[Manipulate[Show[ReleaseHold[CountourPlotHolded], Graphics[OptionValue[Global`LabelFunction] /@ ($LabelList = AppendValue[Posis, SetPrecision[#, OptionValue[Global`LabelPrecision]] & /@ (PlotFunction /. ({x ->#[[1]], y -> #[[2]]} & /@ Posis))])]], {{Posis, {STDLocatorPosition}},Locator, LocatorAutoCreate -> True}]];
   ];

(*
Values for DeltaAlphaHadronic5atMZ and DeltaAlphaLeptonicMZ are \
taken from this paper:
J.~H.~Kuhn and M.~Steinhauser,%``A theory driven analysis of the \
effective QED coupling at M(Z),''
Phys.\ Lett.\  B {\bf 437} (1998) \
425
[arXiv:hep-ph/9802241].
*)

Options[SetDeltaAlpha] = {Global`DeltaAlphaHadronic5AtMZ ->277.45*10^(-4), Global`DeltaAlphaLeptonicAtMZ -> (314.19 + 0.78)*10^(-4), Global`MZ -> 91.1876, Global`Silent->False};
SetDeltaAlpha[opt : OptionsPattern[]] := (SigmaGamma[Global`exp, Global`had5] = OptionValue[Global`DeltaAlphaHadronic5AtMZ]*OptionValue[Global`MZ]^2; 
SigmaGamma[Global`exp, Global`lep] = OptionValue[Global`DeltaAlphaLeptonicAtMZ]*OptionValue[Global`MZ]^2; 
If[! OptionValue[Global`Silent], Print["Assigning values to SigmaGamma[exp,had5] and SigmaGamma[exp,lep]:\nSigmaGamma[exp,had5]=" <> ToString[SigmaGamma[exp, had5]] <> ",\nSigmaGamma[exp,lep]=" <> ToString[SigmaGamma[exp, lep]] <> "."]];)


KeepPower[expr_, var_, listing_] := Module[{templisting, seriesmaxpower, seriestothepower, firstpower, lastpower, minpower, maxpower, tempexpr, coeffcientlist, i},
(*Funktion zum Umrechnen der Externen Potenz in die zugeordnete Listenstelle*)
ExPToLN[n_]:=n*seriestothepower-tempexpr[[4]]+1;

(*DieFunktionkannzweiunterschieldicheArtenvonArgumenten entegennehmen I: EntwicklungbiszurOrdnungmaxpower:*)
If[Head[listing]=!=List,
templisting={listing};];
(*II: Entwicklung von der Ordnung minpower bis maxpower:*)
If[Head[listing]==List,
templisting=Union[listing];];

(*I*)
If[Length[templisting]<2&&Head[templisting[[1]]]==Integer,
maxpower=templisting[[1]];
tempexpr=Series[expr,{var,0,maxpower}];
coeffcientlist=tempexpr[[3]];
seriestothepower=tempexpr[[6]];
firstpower=tempexpr[[4]]/seriestothepower;
lastpower=(tempexpr[[4]]+Length[coeffcientlist]-1)/seriestothepower;
i=1;
coeffcientlist=(If[i++<=ExPToLN[maxpower],#,0])&/@coeffcientlist;
tempexpr=ReplacePart[tempexpr,coeffcientlist,3];
Return[Normal[tempexpr]];
];

(*II*)
If[(Length[templisting]==2)&&(Head[templisting[[1]]]==Integer)&&(Head[templisting[[2]]]==Integer),
minpower=templisting[[1]];
maxpower=templisting[[2]];
tempexpr=Series[expr,{var,0,maxpower}];
coeffcientlist=tempexpr[[3]];
seriestothepower=tempexpr[[6]];
firstpower=tempexpr[[4]]/seriestothepower;
lastpower=(tempexpr[[4]]+Length[coeffcientlist]-1)/seriestothepower;
i=1;
coeffcientlist=(If[ExPToLN[minpower]<=i++<=ExPToLN[maxpower],#,0])&/@coeffcientlist;
tempexpr=ReplacePart[tempexpr,coeffcientlist,3];
Return[Normal[tempexpr]];
];

Return["WrongInputValius!"];
];


(*Creats a graphic box with 4 lines used as subfunction!*)
Options[BoxObject] = {BoxLineProperties -> {Thick, AbsoluteDashing[{1, 1}], Black}};
BoxObject[{x1_List, x2_List}, opt : OptionsPattern[]] := Module[{UL, OR},
   (*UntenLinks*)
   ULx = Min[x1[[1]], x2[[1]]];
   ULy = Min[x1[[2]], x2[[2]]];
   (*Oben Rechts*)
   ORx = Max[x1[[1]], x2[[1]]];
   ORy = Max[x1[[2]], x2[[2]]];
   Return[{Sequence @@ OptionValue[BoxLineProperties], Line[{{ULx, ORy}, {ULx, ULy}, {ORx, ULy}, {ORx, ORy}, {ULx, ORy}}]}];
];

(*Creats a LineTab. Used subfunction!*)
Options[DrawLinesTab] = Union[{LineOffset -> {0, 0}, LineLength -> 0.8, LineProperties -> {Thick, Black}, LayoutBoxes -> False}, Options[BoxObject]];
DrawLinesTab[{x1_List, x2_List}, opt : OptionsPattern[]] := Module[{UL, OR, CurrentOptionValueRule, BoxObjectOptions, LineTabWidth, LineTabHight, CenterTabPoint, StartingLinePoint, EndingLinePoint},
   CurrentOptionValueRule = Rule[#, OptionValue[#]] & /@ (#[[1]] & /@ Options[DrawLinesTab]);
   BoxObjectOptions = FilterRules[CurrentOptionValueRule, Options[BoxObject]];

   (*UntenLinks*)
   ULx = Min[x1[[1]], x2[[1]]];
   ULy = Min[x1[[2]], x2[[2]]];
   (*Oben Rechts*)
   ORx = Max[x1[[1]], x2[[1]]];
   ORy = Max[x1[[2]], x2[[2]]];

   LineTabWidth = ORx - ULx;
   LineTabHight = ORy - ULy;

   CenterTabPoint = {ULx + LineTabWidth/2, ULy + LineTabHight/2};
   StartingLinePoint = CenterTabPoint - {OptionValue[LineLength]*LineTabWidth/2, 0} + LineTabWidth*OptionValue[LineOffset];
   EndingLinePoint = CenterTabPoint + {OptionValue[LineLength]*LineTabWidth/2, 0} + LineTabWidth*OptionValue[LineOffset];

   If[OptionValue[LayoutBoxes],
    Return[{Sequence @@ BoxObject[{x1, x2}, Sequence @@ BoxObjectOptions], Sequence @@ OptionValue[LineProperties], Line[{StartingLinePoint, EndingLinePoint}]}];,
    Return[{Sequence @@ OptionValue[LineProperties], Line[{StartingLinePoint, EndingLinePoint}]}];
   ];
];

(*Creats a LableTab. Used subfunction!*)
Options[DrawLablesTab] = Union[{TextOffset -> {0, 0}, LayoutBoxes -> False}, Options[BoxObject]];
DrawLablesTab[{x1_List, x2_List}, StyleText_, opt : OptionsPattern[]] := Module[{UL, OR, CurrentOptionValueRule, BoxObjectOptions, LableTabWidth, LableTabHight, CenterTabPoint},

   CurrentOptionValueRule = Rule[#, OptionValue[#]] & /@ (#[[1]] & /@ Options[DrawLablesTab]);
   BoxObjectOptions = FilterRules[CurrentOptionValueRule, Options[BoxObject]];

   (*UntenLinks*)
   ULx = Min[x1[[1]], x2[[1]]];
   ULy = Min[x1[[2]], x2[[2]]];
   (*Oben Rechts*)
   ORx = Max[x1[[1]], x2[[1]]];
   ORy = Max[x1[[2]], x2[[2]]];

   LableTabWidth = ORx - ULx;
   LableTabHight = ORy - ULy;

   CenterTabPoint = {ULx + LableTabWidth/2, ULy + LableTabHight/2} + OptionValue[TextOffset];

   If[OptionValue[LayoutBoxes],
    Return[{Text[StyleText, CenterTabPoint], Sequence @@ BoxObject[{x1, x2}, Sequence @@ BoxObjectOptions]}];,
    Return[{Text[StyleText, CenterTabPoint]}];
   ];
];

(*Creates a Plot Legend (Graphic Object)*) 
Options[DrawLegend] = Union[{LegendPosition -> {0, 1}, LegendSize -> {1, 1}, LegendFrame -> False, LegendFrameStyle -> {Thick, AbsoluteDashing[{1, 1}], Black}, TextSpace -> 0.3, LineSpace -> 0.3}, Options[DrawLablesTab], Options[DrawLinesTab]];
DrawLegend[LableList_, options : OptionsPattern[]] := Module[{CurrentOptionValueRule, OptionsDrawLablesTab, OptionsDrawLinesTab, NumberOfRows, ReversedLableList, ListOfLineMasterPoints, ListOfLableMasterPoints, LineTabHight, LineTabWidth, LableTabHight, LableTabWidth, LableTabsList, LineTabsList, TabsList, n},

  CurrentOptionValueRule = Rule[#, OptionValue[#]] & /@ (#[[1]] & /@ Options[DrawLegend]);
  OptionsDrawLablesTab = FilterRules[CurrentOptionValueRule, Options[DrawLablesTab]];
  OptionsDrawLinesTab = FilterRules[CurrentOptionValueRule, Options[DrawLinesTab]];

  NumberOfRows = Length[LableList];
  ReversedLableList = Reverse[LableList];

  (*Calculating x/y-Values of master points*)
  LineTabHight = (OptionValue[LegendSize])[[2]]/NumberOfRows;
  LineTabWidth = (OptionValue[LegendSize])[[1]]*OptionValue[LineSpace];
  LableTabHight = (OptionValue[LegendSize])[[2]]/NumberOfRows;
  LableTabWidth = (OptionValue[LegendSize])[[1]]*OptionValue[TextSpace];

  ListOfLineMasterPoints = {{OptionValue[LegendPosition][[1]], OptionValue[LegendPosition][[2]] + LineTabHight*(# - 1)}, {OptionValue[LegendPosition][[1]] + LineTabWidth, OptionValue[LegendPosition][[2]] + LineTabHight*#}} & /@ Range[NumberOfRows];
  ListOfLableMasterPoints = {{OptionValue[LegendPosition][[1]] + (OptionValue[LegendSize][[1]] - LableTabWidth), OptionValue[LegendPosition][[2]] + LableTabHight*(# - 1)}, {OptionValue[LegendPosition][[1]] + OptionValue[LegendSize][[1]], OptionValue[LegendPosition][[2]] + LableTabHight*#}} & /@ Range[NumberOfRows];

  n = 1;
  LableTabsList = DrawLablesTab[#, ReversedLableList[[n++, 1]], OptionsDrawLablesTab] & /@ ListOfLableMasterPoints;

  n = 1;
  LineTabsList = DrawLinesTab[#, LineProperties -> ReversedLableList[[n++, 2]], OptionsDrawLinesTab] & /@ ListOfLineMasterPoints;

  n = 1;
  TabsList = Flatten[({#, LableTabsList[[n++]]} & /@ LineTabsList)];

  If[OptionValue[LegendFrame],
   TabsList = {Sequence @@ BoxObject[{OptionValue[LegendPosition], OptionValue[LegendPosition] + OptionValue[LegendSize]}, BoxLineProperties -> OptionValue[LegendFrameStyle]], Sequence @@ TabsList};
   ];

  Return[Graphics[TabsList]];
]


End[];