(*Creation of SPS DataSets with SPheno*)
Needs["TQPAT`"];

(*
Definitions of SPS Points and Slopes

see for example:
B.~C.~Allanach {\it et al.},
``The Snowmass points and slopes: Benchmarks for SUSY searches,''
in {\it Proc. of the APS/DPF/DPB Summer Study on the Future of Particle Physics (Snowmass 2001) } ed. N.~Graf,
  Eur.\ Phys.\ J.\  C {\bf 25} (2002) 113
  [arXiv:hep-ph/0202233].
*)

(*SPS1*)
Slope1DataSetRules = {VariableParameter -> x, m0 -> 0.4*x, m12 -> x, A0 -> -0.4*x, tanbeta -> 10, signmu -> 1, ParameterMin -> 100, ParameterMax -> 350, NumberOfPoints -> 51};

(*SPS2*)
Slope2DataSetRules = {VariableParameter -> x, m0 -> 2*x + 850, m12 -> x, A0 -> 0, tanbeta -> 10, signmu -> 1, ParameterMin -> 150, ParameterMax -> 400, NumberOfPoints -> 51};

(*SPS3*)
Slope3DataSetRules = {VariableParameter -> x, m0 -> 0.25*x - 10, m12 -> x, A0 -> 0, tanbeta -> 10, signmu -> 1, ParameterMin -> 250, ParameterMax -> 500, NumberOfPoints -> 51};

(*Points:*)
(*ParameterMin lables the different Sets*)
PointSPS1ADataSetRules = {VariableParameter -> x, m0 -> 100, m12 -> 250, A0 -> -100, tanbeta -> 10, signmu -> 1, ParameterMin -> SPS1A, ParameterMax -> 1, NumberOfPoints -> 1};
PointSPS1APrimeDataSetRules = {VariableParameter -> x, m0 -> 70, m12 -> 250, A0 -> -300, tanbeta -> 10, signmu -> 1, ParameterMin -> SPS1APrime, ParameterMax -> 1,NumberOfPoints -> 1};
PointSPS1BDataSetRules = {VariableParameter -> x, m0 -> 200, m12 -> 400, A0 -> 0, tanbeta -> 30, signmu -> 1, ParameterMin -> SPS1B, ParameterMax -> 1, NumberOfPoints -> 1};
PointSPS2DataSetRules = {VariableParameter -> x, m0 -> 1450, m12 -> 300, A0 -> 0, tanbeta -> 10, signmu -> 1, ParameterMin -> SPS2, ParameterMax -> 1, NumberOfPoints -> 1};
PointSPS3DataSetRules = {VariableParameter -> x, m0 -> 90, m12 -> 400, A0 -> 0, tanbeta -> 10, signmu -> 1, ParameterMin -> SPS3, ParameterMax -> 1, NumberOfPoints -> 1};
PointSPS4DataSetRules = {VariableParameter -> x, m0 -> 400, m12 -> 300, A0 -> 0, tanbeta -> 50, signmu -> 1, ParameterMin -> SPS4, ParameterMax -> 1, NumberOfPoints -> 1};
PointSPS5DataSetRules = {VariableParameter -> x, m0 -> 150, m12 -> 300, A0 -> -1000, tanbeta -> 5, signmu -> 1, ParameterMin -> SPS5, ParameterMax -> 1, NumberOfPoints -> 1};


(*Creation of DataSets*)
SPS1100to350DataSet = CreateDataSet[Slope1DataSetRules];
SPS2150to400DataSet = CreateDataSet[Slope2DataSetRules];
SPS3250to500DataSet = CreateDataSet[Slope3DataSetRules];

SPSPointsDataSet = CreateDataSet[#][[1]] & /@ {PointSPS1ADataSetRules, PointSPS1APrimeDataSetRules, PointSPS1BDataSetRules, PointSPS2DataSetRules, PointSPS3DataSetRules, PointSPS4DataSetRules, PointSPS5DataSetRules};

ExampleDataSetFileList={"SPSPointsDataSet","SPS1100to350DataSet","SPS2150to400DataSet","SPS3250to500DataSet"};

(*Setting Path Option for DisplayDataSets will automaticly set standard Path option of LoadDataSet, SaveDataSet and DeleteDataSetFile to the same value.*)
SetOptions[DisplayDataSets,Path->($TQPATDir<>"/examples/datasets")];

(*Removing existing DataSet files*)
DeleteDataSetFile[#]&/@ExampleDataSetFileLis;


(*Save the created DataSet to file with same name*) 
SaveDataSet[#]&/@{Hold[SPSPointsDataSet],Hold[SPS1100to350DataSet],Hold[SPS2150to400DataSet],Hold[SPS3250to500DataSet]};

(*Shows all files with extension .set in the folder stored in the Path option.*)
DisplayDataSets[]

(*
  Comparisation of created DataSet with package internal DataSets:

  Load the package internal DataSet into math kernel
*)
SetOptions[DisplayDataSets,Path->$TQPATDataSetDir];
LoadDataSet[#,VariableName->(#<>"STD")]&/@DisplayDataSets[];

(*
  If DataSets agree results are 0.
*)
Print["The entries in the following list should be zero:"];
(Plus @@Flatten[ToExpression[#]-ToExpression[#<>"STD"]])&/@ExampleDataSetFileList

