(*This example reproduces the numerical results given in:
  A.~H.~Hoang and C.~J.~Reisser,
  ``On electroweak matching conditions for top pair production at threshold,''
  Phys. Rev. D {\bf 74} (2006) 034002
  [arXiv:hep-ph/0604104].
*)
Needs["TQPAT`"];

(*Loading the numerical values stored in a predefined DataSet*)
LoadDataSet["SMDataSet"];

BornCA[MT_, MH_] := (DoAllNumerics[Union[-1/2*BornHeliL, 1/2*BornHeliR], {#}] &@ReplaceValuesInDataSet[SMDataSet[[1]], {MT2 -> MT^2, Mh02 -> MH^2}])[[1]][[2]];
BornCV[MT_, MH_] := (DoAllNumerics[Union[1/2*BornHeliL, 1/2*BornHeliR], {#}] &@ReplaceValuesInDataSet[SMDataSet[[1]], {MT2 -> MT^2, Mh02 -> MH^2}])[[1]][[2]];
AllEWCA[MT_, MH_] := (DoAllNumerics[Union[-1/2*FullSMHeliL, 1/2*FullSMHeliR], {#}] &@ReplaceValuesInDataSet[SMDataSet[[1]], {MT2 -> MT^2, Mh02 -> MH^2}])[[1]][[2]];
AllEWCV[MT_, MH_] := (DoAllNumerics[Union[1/2*FullSMHeliL, 1/2*FullSMHeliR], {#}] &@ReplaceValuesInDataSet[SMDataSet[[1]], {MT2 -> MT^2, Mh02 -> MH^2}])[[1]][[2]];

(*Calculation of table entries*)
SMDataTable = Prepend[(MT = #[[1]]; MH = #[[2]]; {MT, MH, 10^7*BornCV[MT, MH], 10^7*BornCA[MT, MH], 10^8*AllEWCV[MT, MH], 10^9*AllEWCA[MT, MH], AllEWCV[MT, MH]/BornCV[MT, MH], AllEWCA[MT, MH]/BornCA[MT, MH]}) & /@ Flatten[Outer[List, {170, 172.5, 175}, {115, 150, 200, 1000}], 1], {"m_t(GeV)", "m_H(GeV)", "C_V^{Born}(10^{-7}(GeV^{-2})","C_A^{Born}(10^{-7}(GeV^{-2})","C_V^{EW}(10^{-8}(GeV^{-2})","C_A^{EW}(10^{-9}(GeV^{-2})","C_V^{EW}/{C_V^{Born}","C_A^{EW}/C_A^{Born}"}];

CreateDirectory["tables"];
Export[$TQPATDir <> "/examples/tables/SMDataTable.tbl",SMDataTable,"Table"];

(*
Generate a Higgs-Mass dependent DataSet from first element of SMDataSet.
The mass of the SM Higgs-Boson is given by Mh0.
*)
SetOptions[CreateSimpleDataSet, VariableParameter -> x, ParameterMin -> 100, ParameterMax -> 1000, NumberOfPoints -> 26];
HiggsMassDependentSMDataSet = CreateSimpleDataSet[SMDataSet[[1]], {Mh02 -> x^2}];

(*
Numerical evaluation of analytic results 
(using finite fermion masses in the VB SE contributions) 
 for the given DataSet
*)
HiggsMassDependentDeltaCSMEW = DoAllNumerics[FullSMDeltaC, HiggsMassDependentSMDataSet];
(*
Numerical evaluation of analytic results 
(using SigmaGamma[exp,had5] and SigmaGamma[exp,lep] to enable the vanishing mass limit for light fermions)
*)
HiggsMassDependentDeltaCSMEWNoLFMs = DoAllNumerics[FullSMNoLFMsDeltaC, HiggsMassDependentSMDataSet];

(*The special Options in the Plot function do only work with Mathematica 6.0 or higher!*)
Needs["PlotLegends`"]

TQPATDashed = AbsoluteDashing[{7, 7}];

PlotSM = Plot[{Interpolation[HiggsMassDependentDeltaCSMEW][x],Interpolation[HiggsMassDependentDeltaCSMEWNoLFMs][x]}, {x, 100, 1000}, 
  Frame -> True, 
  AxesOrigin -> {100, 0}, 
  FrameLabel -> {Style["\!\(\*SubscriptBox[\"m\", \"H\"]\)(GeV)", 18],Style[\[CapitalDelta]^EW, 18]}, 
  ImageSize -> 500, 
  PlotStyle -> {{Black, Thick, TQPATDashed},{Black, Thick}}, 
  LabelStyle -> Directive[Italic], 
  FrameStyle -> 16, 
  PlotRange -> {0, 0.2},
  PlotLegend -> {Text[Style["\!\(\*SubscriptBox[\"m\", SubscriptBox[\"F\", \"light\"]]\)\[NotEqual]0", Black, Italic, 12]], Text[Style["\!\(\*SubsuperscriptBox[\"\[CapitalDelta]\[Alpha]\", \"had\", \"5\"]\)(\!\(\*SubscriptBox[\"m\", \"z\"]\)),\!\(\*SubscriptBox[\"\[CapitalDelta]\[Alpha]\", \"lep\"]\)(\!\(\*SubscriptBox[\"m\", \"z\"]\)) \[RightArrow] \!\(\*SubscriptBox[\"m\", SubscriptBox[\"F\", \"light\"]]\)=0", Black, Italic, 12]]},
  LegendPosition -> {-0.3, 0.3},
  LegendSize -> {1.2, 0.2},
  LegendTextOffset -> Automatic,
  LegendTextSpace -> 8,
  LegendShadow -> None,
  LegendBorder -> None,
  LegendBorderSpace -> 0
];

CreateDirectory[$TQPATDir <> "/examples/pics"];
Export[$TQPATDir <> "/examples/pics/DeltaEWSMofMH.eps", PlotSM];
